;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Info Box DX
;; Code and comments from Sonikku's Info Box Disassembly and Dispari Scuro's NPCv3.0 sprite
;; Copypasted together by yoshicookiezeus
;;
;; The below description is also from the NPC sprite.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Extra Info:
;; 02 = Message 1
;; 03 = Message 2
;;
;; After you determine which message the sprite will read from, you need to place it
;; in a good X position on the screen. When placing the sprite, the X position will
;; determine part one of which level the sprite actually reads from. There are 16
;; unique X positions possible per screen, which allows up to 16 unique messages.
;; The number starts over on the next screen. So if your sprite is in the first
;; position but on page 4, it's still considered 0 (position 1). Think of it like
;; the original messagebox, which changed its message based on its X position.
;; Only instead of only displaying one of two messages, you display one of 16.
;; In commbination with the Extra Info, that's one of 32.
;;
;; The reason for the X position is to determine which color on the palette to read
;; from to use. If you set the X position for a sprite to 0, it will use the first
;; color on the palette as a reference. Therefore, you can set color 0 to color 24
;; to make the sprite read messages from level 24. This DOES mean that you can use
;; ExAnimation to palette animate this color and generate "random" messages! Please
;; note the special handling for levels over 24. For any level number over 24,
;; subtract DC from the level number. For example, level 105 would be 29. Note
;; that to properly use the palette, you have to set the color so that its SNES
;; RGB value is the level you want it to be, times 100. So if you want to read
;; from level 20, the SNES RGB value needs to be 2000. The easiest way to do this
;; is to set an ExAnimation palette animation of 2000 and then check the actual
;; palette to see what to paste. 2000 for instance is 0 red, 0 green, 64 blue.
;;
;; NOTE: If you use palette ExAnimation on the colour that determines which message
;; to read (even if it's just one static colour), you have to use the
;; Type: Palette + Working option!
;;
;; Here are some examples of how these three varibles come together. For the
;; sake of this example, we will assume the sprite is still reading from palette E.
;;
;; Example 1:
;; Extra Info is 2. X position is 6 (on screen 1). Palette E6 is 1200. This means the
;; sprite will display message 1 from level 12. X position is 6 so it reads from E6.
;; F6 says to read from level 12.
;;
;; Example 2:
;; Extra Info is 3. X position is 42 (on screen 3). Palette EB is 3200. This means the
;; sprite will display message 2 from level 10E. X position is 11 (11th spot on screen
;; 3). EB (B = 11) says to read from level 10E (32 + DC = 10E).
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite constants - feel free to change these
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	!Tile			= $C0
	!MessagePalette		= $E0

YPos:	db $00,$04,$07,$08,$08,$07,$04,$00,$00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ROM and RAM defines - don't mess with these
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	!RAM_ScreenBndryYLo	= $1C
	!RAM_ScreenBndryYHi	= $1D
	!RAM_SpriteState	= $C2
	!RAM_SpriteXLo		= $E4
	!OAM_Tile		= $0302

	!RAM_ExtraBits		= $7FAB10

	!InvisBlkMainRt		= $01B44F

	!RAM_SpriteXBlock	= $1504

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	print "INIT ",pc
	LDA !RAM_SpriteXLo,x		;\ Grab X position
	LSR A				; |
	LSR A				; |
	LSR A				; |
	LSR A				; |
	STA !RAM_SpriteXBlock,x		;/ Save for later
	RTL

	print "MAIN ",pc
	PHB
	PHK
	PLB
	JSR Main
	PLB
	RTL     

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Main:
	JSL !InvisBlkMainRt		; Load invisible solid block routine.
	JSR SubOffScreenX3		; Handle offscreen situation.
	JSR GetDrawInfo			; Get draw info and whatever.
	LDA $1558,x			; If timer for sprite..
	CMP #$01			; isn't 1..
	BNE Code038D93			; Set Y position.
	LDA #$22			; Play Sound.
	STA $1DFC			; Store it.
	STZ $1558,x			; Restore timer.
	STZ !RAM_SpriteState,x		; Make it so it can be hit again.
	LDA #!MessagePalette		;\
	CLC				; | Figure out which palette to use for level
	ADC !RAM_SpriteXBlock,x		;/
	PHX
	REP #$30
	AND.w #$00FF
	ASL A
	TAX
	LDA $0704,x
	SEP #$30
	PLX
	STA $08				; Store so we know which level to read from
	
	LDA !RAM_ExtraBits,x		;\ Get Extra Info
	AND #$04			;/
	BNE Scratch1			;\
	LDA #$01			; | Extra Info is kinda wacky
	BRA Scratch2			;/
	
Scratch1:
	LDA #$02
	
Scratch2:
	STA $09				; Scratch RAM is a go
	
	LDA $08				; \ This allows you to read from any level
	STA $13BF			; | you want by tricking the game into thinking
					; | the level's number is something other than
					; / what it actually is.
	
	LDA $09
	STA $1426			; Display message specified

Code038D93:
	LDA $1558,x			; I just took this code out of all.log.
	LSR				; I didn't bother commenting it..
	TAY				; Since I don't really have the patience to.
	LDA !RAM_ScreenBndryYLo		; This code wasn't really documented..
	PHA				; In all.log to begin with..
	CLC				; So I only know this code sets the Y position..
	ADC YPos,y			; Of the tile..
	STA !RAM_ScreenBndryYLo		; When Mario hits this sprite..
	LDA !RAM_ScreenBndryYHi		; from the bottom..
	PHA				; ..
	ADC #$00			; ..
	STA !RAM_ScreenBndryYHi		; ..
	JSL $0190B2			; Load generic graphics routine.
	LDY $15EA,x			; Load sprite OAM.
	LDA #!Tile			; Load tile number..
	STA !OAM_Tile,y			; And store it.
	PLA				; Pull A.
	STA !RAM_ScreenBndryYHi		; Store to Layer 1 Y position (High byte).
	PLA				; Pull A.
	STA !RAM_ScreenBndryYLo		; And store to Layer 1 Y position (Low byte).
	RTS				; Return.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GetDrawInfo
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:	db $0C,$1C
Table2:	db $01,$02

GetDrawInfo:
	STZ $186C,x		; reset sprite offscreen flag, vertical
	STZ $15A0,x		; reset sprite offscreen flag, horizontal
	LDA $E4,x		;\
	CMP $1A			; | set horizontal offscreen if necessary
	LDA $14E0,x		; |
	SBC $1B			; |
	BEQ .OnScreenX		; |
	INC $15A0,x		;/

.OnScreenX
	LDA $14E0,x		;\
	XBA			; |
	LDA $E4,x		; |
	REP #$20		; |
	SEC			; |
	SBC $1A			; | mark sprite invalid if far enough off screen
	CLC			; |
	ADC #$0040		; |
	CMP #$0180		; |
	SEP #$20		; |
	ROL A			; |
	AND #$01		; |
	STA $15C4,x		;/ 
	BNE .Invalid		; 

	LDY #$00		;\ set up loop:
	LDA $1662,x		; | 
	AND #$20		; | if not smushed (1662 & 0x20), go through loop twice
	BEQ .OnScreenLoop	; | else, go through loop once
	INY			;/ 
.OnScreenLoop
	LDA $D8,x		;\ 
	CLC			; | set vertical offscreen if necessary
	ADC Table1,y		; |
	PHP			; |
	CMP $1C			; | (vert screen boundary)
	ROL $00			; |
	PLP			; |
	LDA $14D4,x		; | 
	ADC #$00		; |
	LSR $00			; |
	SBC $1D			; |
	BEQ .OnScreenY		; |
	LDA $186C,x		; | (vert offscreen)
	ORA Table2,y		; |
	STA $186C,x		; |
.OnScreenY
	DEY			; |
	BPL .OnScreenLoop	;/

	LDY $15EA,x		; get offset to sprite OAM
	LDA $E4,x		;\ 
	SEC			; | 
	SBC $1A			; | $00 = sprite x position relative to screen boarder
	STA $00			;/ 
	LDA $D8,x		;\ 
	SEC			; | 
	SBC $1C			; | $01 = sprite y position relative to screen boarder
	STA $01			;/ 
	RTS			; return

.Invalid
	PLA			;\ return from *main gfx routine* subroutine...
	PLA			; |    ...(not just this subroutine)
	RTS			;/
	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SubOffScreen
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
Spr_T12:
	db $40,$B0
Spr_T13:
	db $01,$FF
Spr_T14:
	db $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
	db $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
Spr_T15:
	db $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
	db $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SubOffScreenX1:
	LDA #$02		;\ entry point of routine determines value of $03
	BRA Store03		; | (table entry to use on horizontal levels)
SubOffScreenX2:
	LDA #$04		; | 
	BRA Store03		; |
SubOffScreenX3:
	LDA #$06		; |
	BRA Store03		; |
SubOffScreenX4:
	LDA #$08		; |
	BRA Store03		; |
SubOffScreenX5:
	LDA #$0A		; |
	BRA Store03		; |
SubOffScreenX6:
	LDA #$0C		; |
	BRA Store03		; |
SubOffScreenX7:
	LDA #$0E		; |
Store03:
	STA $03			; |            
	BRA StartSub		; |
SubOffScreenX0:
	STZ $03			;/

StartSub:
	JSR SubIsOffScreen	;\ if sprite is not off screen, return
	BEQ .Return		;/
	LDA $5B			;\  goto VERTICAL_LEVEL if vertical level
	AND #$01		; |
	BNE .VerticalLevel	;/     
	LDA $D8,x		;\
	CLC			; | 
	ADC #$50		; | if the sprite has gone off the bottom of the level...
	LDA $14D4,x		; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
	ADC #$00		; | 
	CMP #$02		; | 
	BPL .EraseSprite	;/    ...erase the sprite
	LDA $167A,x		;\ if "process offscreen" flag is set, return
	AND #$04		; |
	BNE .Return		;/
	LDA $13
	AND #$01
	ORA $03
	STA $01
	TAY
	LDA $1A
	CLC
	ADC Spr_T14,y
	ROL $00
	CMP $E4,x
	PHP
	LDA $1B
	LSR $00
	ADC Spr_T15,y
	PLP
	SBC $14E0,x
	STA $00
	LSR $01
	BCC .Label31
	EOR #$80
	STA $00
.Label31
	LDA $00
	BPL .Return
.EraseSprite
	LDA $14C8,x		;\ if sprite status < 8, permanently erase sprite
	CMP #$08		; |
	BCC .KillSprite		;/    
	LDY $161A,x
	CPY #$FF
	BEQ .KillSprite
	LDA #$00
	STA $1938,y
.KillSprite
	STZ $14C8,x		; erase sprite
.Return
	RTS			; return

.VerticalLevel
	LDA $167A,x		;\ if "process offscreen" flag is set, return
	AND #$04		; |
	BNE .Return		;/
	LDA $13			;\
	LSR A			; | 
	BCS .Return		;/
	LDA $E4,x		;\ 
	CMP #$00		; | if the sprite has gone off the side of the level...
	LDA $14E0,x		; |
	SBC #$00		; |
	CMP #$02		; |
	BCS .EraseSprite	;/  ...erase the sprite
	LDA $13
	LSR A
	AND #$01
	STA $01
	TAY
	LDA $1C
	CLC
	ADC Spr_T12,y
	ROL $00
	CMP $D8,x
	PHP
	LDA $001D
	LSR $00
	ADC Spr_T13,y
	PLP
	SBC $14D4,x
	STA $00
	LDY $01
	BEQ .Label38
	EOR #$80
	STA $00
.Label38
	LDA $00
	BPL .Return
	BMI .EraseSprite

SubIsOffScreen:
	LDA $15A0,x		;\ if sprite is on screen, accumulator = 0 
	ORA $186C,x		; |  
	RTS			;/ return